require "import"
import "android.net.Uri"
import "android.content.Intent"
import "android.provider.ContactsContract"
import "android.widget.*"
import "android.view.*"
import "java.util.*"
import "android.content.pm.PackageManager"
import "android.os.Build"
import "android.Manifest"
import "android.app.AlertDialog"
import "android.text.ClipboardManager"

-- Permission codes
local CONTACTS_PERMISSION_CODE = 100
local CALL_PERMISSION_CODE = 101

-- Required permissions
local permissions = {
  ["READ_CONTACTS"] = Manifest.permission.READ_CONTACTS,
  ["CALL_PHONE"] = Manifest.permission.CALL_PHONE
}

-- Globals
local contactsList = ArrayList()
local numbersList = {}

-- Layout
mainLayout = {
  LinearLayout;
  orientation = "vertical";
  padding = "10dp";
  {
    Button;
    text = "➕ Create New Contact";
    layout_width = "match_parent";
    layout_height = "wrap_content";
    onClick = function()
      local intent = Intent(Intent.ACTION_INSERT)
      intent.setType(ContactsContract.Contacts.CONTENT_TYPE)
      activity.startActivity(intent)
    end
  };
  {
    TextView;
    id = "titleText";
    textSize = "20sp";
    textColor = 0xFF000000;
    paddingTop = "10dp";
    gravity = "center";
  };
  {
    TextView;
    id = "countText";
    textSize = "16sp";
    textColor = 0xFF444444;
    paddingBottom = "8dp";
    gravity = "center";
  };
  {
    TextView;
    text = "📋 Here are your saved contacts:";
    textSize = "15sp";
    textColor = 0xFF555555;
    padding = "8dp";
  };
  {
    ListView;
    id = "listView";
    layout_width = "match_parent";
    layout_height = "match_parent";
    dividerHeight = "1";
  };
}

-- Set layout
activity.setContentView(loadlayout(mainLayout))
titleText.setText("📱 Welcome to Contact Manager")
countText.setText("Loading contacts...")

-- Permission check
function hasPermission(permission)
  if Build.VERSION.SDK_INT >= 23 then
    return activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED
  end
  return true
end

-- Request permission
function requestPermission(permission, code)
  if Build.VERSION.SDK_INT >= 23 then
    activity.requestPermissions({permission}, code)
  end
end

-- Handle permission result
function onRequestPermissionsResult(requestCode, permissions, grantResults)
  if requestCode == CONTACTS_PERMISSION_CODE and grantResults[1] == PackageManager.PERMISSION_GRANTED then
    loadContacts()
  elseif requestCode == CALL_PERMISSION_CODE and grantResults[1] == PackageManager.PERMISSION_GRANTED then
    -- Permission granted for call
  else
    Toast.makeText(activity, "Permission Denied", Toast.LENGTH_SHORT).show()
  end
end

-- Load and display contacts
function loadContacts()
  contactsList.clear()
  numbersList = {}
  local cursor = activity.getContentResolver().query(
    ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
    nil, nil, nil,
    ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " ASC"
  )
  local count = 0
  while cursor.moveToNext() do
    local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
    local number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
    contactsList.add(name .. " (" .. number .. ")")
    table.insert(numbersList, number)
    count = count + 1
  end
  cursor.close()

  countText.setText("📇 Total Contacts: " .. tostring(count))

  local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, contactsList)
  listView.setAdapter(adapter)

  -- Tap listener for contact actions
  listView.setOnItemClickListener(AdapterView.OnItemClickListener {
    onItemClick = function(_, _, position)
      local phone = numbersList[position + 1]
      showOptionsMenu(phone)
    end
  })
end

-- Options menu for contact
function showOptionsMenu(phoneNumber)
  local items = {
    "📞 Call " .. phoneNumber,
    "📋 Copy Number: " .. phoneNumber,
    "📤 Share Number: " .. phoneNumber,
    "💬 Send Message to " .. phoneNumber,
    "ℹ️ View Contact Details",
    "❌ Cancel"
  }

  AlertDialog.Builder(activity)
    .setTitle("Choose an option for " .. phoneNumber)
    .setItems(items, function(_, index)
      if index == 0 then
        if not hasPermission(permissions["CALL_PHONE"]) then
          requestPermission(permissions["CALL_PHONE"], CALL_PERMISSION_CODE)
        else
          makePhoneCall(phoneNumber)
        end
      elseif index == 1 then
        local cm = activity.getSystemService(activity.CLIPBOARD_SERVICE)
        cm.setText(phoneNumber)
        Toast.makeText(activity, "Number copied", 0).show()
      elseif index == 2 then
        local shareIntent = Intent(Intent.ACTION_SEND)
        shareIntent.setType("text/plain")
        shareIntent.putExtra(Intent.EXTRA_TEXT, phoneNumber)
        activity.startActivity(Intent.createChooser(shareIntent, "Share via"))
      elseif index == 3 then
        local smsIntent = Intent(Intent.ACTION_VIEW)
        smsIntent.setData(Uri.parse("sms:" .. phoneNumber))
        activity.startActivity(smsIntent)
      elseif index == 4 then
        local viewIntent = Intent(Intent.ACTION_VIEW)
        viewIntent.setData(Uri.parse("tel:" .. phoneNumber))
        activity.startActivity(viewIntent)
      end
    end)
    .show()
end

-- Call a number
function makePhoneCall(phoneNumber)
  local callIntent = Intent(Intent.ACTION_CALL)
  callIntent.setData(Uri.parse("tel:" .. phoneNumber))
  activity.startActivity(callIntent)
end

-- Start contact loading
if not hasPermission(permissions["READ_CONTACTS"]) then
  requestPermission(permissions["READ_CONTACTS"], CONTACTS_PERMISSION_CODE)
else
  loadContacts()
end